Package: metaRange
Title: Framework to Build Mechanistic and Metabolic Constrained Species
        Distribution Models
Version: 1.1.4
Authors@R: 
    c(person(given = "Stefan",
             family = "Fallert",
             role = c("aut", "cre", "cph"),
             email = "srfallert@gmail.com",
             comment = c(ORCID = "0000-0002-9939-4219")),
      person(given = "Lea",
             family = "Li",
             role = c("aut", "cph"),
             comment = c("Implemented the first version of the metabolic scaling")),
      person(given = "Juliano",
             family = "Sarmento Cabral",
             role = c("aut", "cph", "ths"),
             email = "j.sarmentocabral@bham.ac.uk",
             comment = c(ORCID = "0000-0002-0116-220X")),
      person(given = "Tyler",
             family = "Morgan-Wall",
             role = c("ctb", "cph"),
             comment = c(ORCID = "0000-0002-3131-3814")),
      person(given = "Bavarian Ministry of Science and Arts (bayklif)",
             role = c("fnd")),
      person(given = "Deutsche Bundesstiftung Umwelt (DBU)",
             role = c("fnd")))
Description: Build spatially and temporally explicit
    process-based species distribution models, that can include an arbitrary
    number of environmental factors, species and processes including metabolic
    constraints and species interactions. The focus of the package is simulating
    populations of one or multiple species in a grid-based landscape and studying
    the meta-population dynamics and emergent patterns that arise from the
    interaction of species under complex environmental conditions. It
    provides functions for common ecological processes such as
    negative exponential, kernel-based dispersal (see
    Nathan et al. (2012) <doi:10.1093/acprof:oso/9780199608898.003.0015>),
    calculation of the environmental suitability based on cardinal values (
    Yin et al. (1995) <doi:10.1016/0168-1923(95)02236-Q>, simplified by
    Yan and Hunt (1999) <doi:10.1006/anbo.1999.0955> see eq: 4), reproduction in
    form of an Ricker model (see Ricker (1954) <doi:10.1139/f54-039> and
    Cabral and Schurr (2010) <doi:10.1111/j.1466-8238.2009.00492.x>),
    as well as metabolic scaling based on the metabolic theory of ecology
    (see Brown et al. (2004) <doi:10.1890/03-9000> and
    Brown, Sibly and Kodric-Brown (2012)
    <doi:10.1002/9781119968535.ch>).
License: GPL-3
Copyright: see inst/COPYRIGHTS
URL: https://metaRange.github.io/metaRange/
BugReports: https://github.com/metaRange/metaRange/issues
Depends: R (>= 3.5.0)
Imports: Rcpp (>= 1.0.10), terra, R6, checkmate, grDevices, graphics,
        utils
Suggests: knitr, rmarkdown, tinytest
VignetteBuilder: knitr
Encoding: UTF-8
RoxygenNote: 7.2.3
LinkingTo: Rcpp, RcppArmadillo
NeedsCompilation: yes
Packaged: 2024-02-09 11:51:34 UTC; srfall
Author: Stefan Fallert [aut, cre, cph]
    (<https://orcid.org/0000-0002-9939-4219>),
  Lea Li [aut, cph] (Implemented the first version of the metabolic
    scaling),
  Juliano Sarmento Cabral [aut, cph, ths]
    (<https://orcid.org/0000-0002-0116-220X>),
  Tyler Morgan-Wall [ctb, cph] (<https://orcid.org/0000-0002-3131-3814>),
  Bavarian Ministry of Science and Arts (bayklif) [fnd],
  Deutsche Bundesstiftung Umwelt (DBU) [fnd]
Maintainer: Stefan Fallert <srfallert@gmail.com>
Repository: CRAN
Date/Publication: 2024-02-09 12:20:04 UTC
Built: R 4.6.0; aarch64-apple-darwin23; 2026-04-25 20:21:04 UTC; unix
Archs: metaRange.so.dSYM
