# Generated by roxygen2: do not edit by hand

S3method(plot,metaRangeEnvironment)
S3method(plot,metaRangeSimulation)
S3method(plot,metaRangeSpecies)
S3method(print,metaRangeVariableStorage)
S3method(summary,metaRangeSimulation)
S3method(summary,metaRangeSpecies)
export(calculate_dispersal_kernel)
export(calculate_normalization_constant)
export(calculate_suitability)
export(create_simulation)
export(dispersal)
export(metaRangeEnvironment)
export(metaRangePriorityQueue)
export(metaRangeProcess)
export(metaRangeSimulation)
export(metaRangeSpecies)
export(metabolic_scaling)
export(negative_exponential_function)
export(ricker_allee_reproduction_model)
export(ricker_reproduction_model)
export(save_species)
export(set_verbosity)
importFrom(R6,R6Class)
importFrom(Rcpp,evalCpp)
importFrom(checkmate,assert_function)
importFrom(terra,writeRaster)
importFrom(utils,write.csv)
useDynLib(metaRange)
